@@ -20,7 +20,7 @@ from flexgen.opt_config import OptConfig, get_opt_config, download_opt_weights
 from flexgen.pytorch_backend import (TorchDevice, TorchDisk, TorchLink,
     TorchMixedDevice, DeviceType, general_copy, fix_recursive_import)
 from flexgen.timer import timers
-from flexgen.utils import (Task, Env, GB, T, ValueHolder,
+from flexgen.utils import (Task, ExecutionEnv, GB, T, ValueHolder,
     array_1d, array_2d, array_3d, str2bool, project_decode_latency,
     torch_mem_stats, torch_dtype_to_np_dtype, write_benchmark_log,
     read_benchmark_log)
@@ -580,7 +580,13 @@ class TransformerLayer:
 
 
 class OptLM:
-    def __init__(self, config, env, path, policy):
+    def __init__(self,
+                 config: Union[str, OptConfig],
+                 env: ExecutionEnv,
+                 path: str,
+                 policy: Policy):
+        if isinstance(config, str):
+            config = get_opt_config(config)
         self.config = config
         self.env = env
         self.path = path
@@ -628,6 +634,7 @@ class OptLM:
         self.attention_mask = array_1d(num_gpu_batches, ValueHolder)
 
         self.task = None
+        self.init_all_weights()
 
     def set_task(self, task):
         self.task = task
@@ -762,7 +769,12 @@ class OptLM:
             left, right = k * gpu_batch_size, (k + 1) * gpu_batch_size
             ids = self.hidden[i][j][k].pop().data.detach().cpu().numpy()
             pos = self.task.prompt_len + i
-            self.output_ids[left:right, pos:pos+1] = ids
+            if self.task.stop:
+                self.output_ids[left:right, pos:pos+1] = np.where(
+                    self.stopped, self.config.pad_token_id, ids)
+                self.stopped = np.logical_or(self.stopped, ids == self.task.stop)
+            else:
+                self.output_ids[left:right, pos:pos+1] = ids
         else:  # move to home
             x = self.hidden[i][j][k]
             if x.val:  # x may already be moved due to overlapping
@@ -835,7 +847,9 @@ class OptLM:
         self.execute_gen_len = task.cut_gen_len if task.cut_gen_len else task.gen_len
 
         # Output token ids
-        self.output_ids = np.ones((len(task.inputs), prompt_len + gen_len), dtype=np.int32)
+        self.output_ids = np.full((len(task.inputs), prompt_len + gen_len),
+            self.config.pad_token_id, dtype=np.int32)
+        self.stopped = np.zeros((len(task.inputs), 1), dtype=bool)
         self.output_ids[:, :prompt_len] = np.asarray(task.inputs)
         assert gpu_batch_size * num_gpu_batches == len(task.inputs)
 
@@ -1013,7 +1027,7 @@ class OptLM:
                 self.sync()
             timers("generate").stop()
 
-            if self.output_ids[0, self.task.prompt_len + i] == self.task.stop:
+            if self.task.stop and np.all(self.stopped):
                 break
 
     def generation_loop_overlap_multi_batch(self):
@@ -1130,6 +1144,9 @@ class OptLM:
             else:
                 timers("generate").costs.append(self.num_layers * batch_cost)
 
+    def __del__(self):
+        self.delete_all_weights()
+
 
 def get_filename(args):
     model_size = args.model.split('-')[-1]
@@ -1170,7 +1187,7 @@ def run_flexgen(args):
     gpu = TorchDevice("cuda:0")
     cpu = TorchDevice("cpu")
     disk = TorchDisk(args.offload_dir)
-    env = Env(gpu=gpu, cpu=cpu, disk=disk, mixed=TorchMixedDevice([gpu, cpu, disk]))
+    env = ExecutionEnv(gpu=gpu, cpu=cpu, disk=disk, mixed=TorchMixedDevice([gpu, cpu, disk]))
 
     policy = Policy(args.gpu_batch_size, args.num_gpu_batches,
                     args.percent[0], args.percent[1],
@@ -1187,31 +1204,28 @@ def run_flexgen(args):
     assert not (args.compress_cache and args.attn_sparsity < 1.0), "Not implemented"
 
     opt_config = get_opt_config(args.model)
-    model = OptLM(opt_config, env, args.path, policy)
     cache_size = opt_config.cache_bytes(num_prompts, prompt_len + gen_len)
     hidden_size = opt_config.hidden_bytes(num_prompts, prompt_len + gen_len)
     print(f"model size: {opt_config.model_bytes()/GB:.3f} GB, "
           f"cache size: {cache_size/GB:.3f} GB, "
           f"hidden size (prefill): {hidden_size/GB:.3f} GB")
 
+    print("init weight...")
+    model = OptLM(opt_config, env, args.path, policy)
+
     try:
-        print("warmup - init weights")
-        model.init_all_weights()
         print("warmup - generate")
         output_ids = model.generate(
             warmup_inputs, max_new_tokens=1, verbose=args.verbose)
 
-        # Benchmark
         print("benchmark - generate")
         timers("generate").reset()
         output_ids = model.generate(
             inputs, max_new_tokens=args.gen_len,
             debug_mode=args.debug_mode, cut_gen_len=cut_gen_len, verbose=args.verbose)
         costs = timers("generate").costs
-        print("benchmark - delete weights")
-        model.delete_all_weights()
     finally:
-        disk.close_copy_threads()
+        env.close_copy_threads()
 
     # Log output
     prefill_latency = costs[0]
