@@ -28,17 +28,29 @@ class Task:
 
     do_sample: bool
     temperature: float
-    stop: int
+    stop: Optional[int]
 
 
 @dataclasses.dataclass(frozen=True)
-class Env:
+class ExecutionEnv:
     """Hardware environment."""
     gpu: Any = None
     cpu: Any = None
     disk: Any = None
     mixed: Any = None
 
+    @classmethod
+    def create(cls, offload_dir):
+        # fix recursive import
+        from flexgen.pytorch_backend import TorchDevice, TorchDisk, TorchMixedDevice
+        gpu = TorchDevice("cuda:0")
+        cpu = TorchDevice("cpu")
+        disk = TorchDisk(offload_dir)
+        return cls(gpu=gpu, cpu=cpu, disk=disk, mixed=TorchMixedDevice([gpu, cpu, disk]))
+
+    def close_copy_threads(self):
+        self.disk.close_copy_threads()
+
 
 @dataclasses.dataclass(frozen=True)
 class BenchmarkResult:
